%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                               %
%% Übung: deleteall/3                                           %
% Schreibe ein Prädikat deleteall/3, welches alle Vorkommen     %
% eines Elements aus einer Liste löscht.                        %
%                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%
%% Erläuterung:
% Unser Ziel ist es, wie oben beschrieben, ein Prädiktat zu definieren das alle Vorkommen eines Elements aus einer Liste löscht. 
% Nehmen wir die Menge A, das Element x und es gilt x ist in A und das beliebig oft, bzw. auch kein mal. 
% Dann gilt für unsere Ergebnis-Menge C, dass x nicht in C ist. Also C = A ohne x.
% Formal: C = {a | a in A und a nicht gleich x} [Vorsicht: a ist eine Variable, ein beliebiges Element in A]
% Daraus ergibt sich für uns auf der Aufgabenstellung folgende Belegung der Argumente für delete_sublist/3:
% delete_sublist(x,A,C)


%
%% Korollar:
% In der Vorlesung wurde das Prädikat delete/3 bereits vorgestellt, welches nur eine Okkurrenz des Elementes 
% aus der gegebenen Liste entfernt. Daher liegt es nahe, dass der Aufbau der beiden Funktionen identisch sein wird, 
% wenn deleteall/3 nicht sogar nur eine Erweiterung von delete/3 darstellt. So sollten wir zuerst delete/3 verstehen,
% bevor wir eine adäquate Lösung für deleteall/3 implementieren.
% Folgend das Prädikat:

% Ankerklausel: 
% Das Element X steht nun als Head in unsere Liste (Element gefunden) 
% und der Tail ist identisch mit dem Ausgabe Tail. (erfolgreicher Durchlauf)
delete(X,[X|T],T).

% Hauptaufruf, inklusive Rekursion:
% Im Prädikatsaufruf teilen wir die Eingabe, sowie Ausgabe Liste in Head und Tail auf.
% Wichtig ist hier, dass das die Heads identisch sind. Dies gilt nicht für die Tails, 
% da in T1 das Element X in unserer Ankerklausel entfernt wird und T2 das Ergebnis darstellt.
delete(X,[H|T1],[H|T2]):-
    delete(X,T1,T2).     % rekursiver Aufruf mit den Tails der Listen


%
%%% Lösung:
% Wir haben nun die Vorgehensweise von delete/3 verstanden und überlegen uns wie wir das Prädikat erweitern müssen. 
% Uns wird bewusst, wenn wir bei der Ankerklausel von delete/3 nicht aufhören, sondern die Liste weiter verarbeiten
% bis wir alle Elemente geprüft haben, sollten wir auf eine nachvollziehbare Lösung kommen.

% Ankerklausel: 
% Wir sind alle Elemente der Eingabe, Ausgabeliste durchlaufen. (erfolgreicher Durchlauf)
delete_all(_,[],[]).

% Rekursiver Aufruf: 
% Dies ist die Ankerklausel von delete/3. Das Element wurde gefunden und entfernt.
% Der Unterschied ist aber, dass wir weiter Suchen bis wir bei unserer neuen Ankerklausel ankommen.
delete_all(E,[E|T],L2):-
     delete_all(E,T,L2).      % rekursiver Aufruf mit den Tails der Listen abzüglich des entfernten Elements

% Hauptaufruf, inklusive Rekursion:
delete_all(E,[H|T1],[H|T2]):-
     E\=H,                    % Prüfen, ob der Head gleich dem gesuchten Element ist, wenn ja siehe Rekursiver Aufruf oben
     delete_all(E,T1,T2).     % rekursiver Aufruf mit den Tails der Listen
     
     
%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.

% ?- deleteall(b,[a,b,c,d],[a,c,d]).
% -> true.

% ?- deleteall(1,[1,2,1,2,3,1,2,3,4],[2,2,3,2,3,4]).
% -> true.

% ?- deleteall(1,[a,b,c],[a,b,c]).
% -> true.

% ?- deleteall(1,[1,1,1,1],[]).
% -> true.


%
%% Weitere Informationen:

% -> (hilfreiche) Diskussion über delete_all/3: https://stackoverflow.com/questions/12175377/deleting-all-occurrences-of-an-element-from-a-list
