%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                       %
% Übung: delete_sublist/3                                               %
% Schreibe ein Prädikat delete_sublist/3, das drei Listen als Argumente %
% nimmt und gelingt, wenn die dritte Liste das Resultat ist, das man    %
% erhält, wenn man alle Vorkommen von Elementen der ersten Liste aus    %
% der zweiten Liste löscht.                                             %                                    
%                                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
%% Erläuterung:
% Unser Ziel ist es, mengentheoretisch gesehen, die Operation der Differenz zu implementieren.
% Nehmen wir die Mengen A und B, dann ist die Differenzmenge C = A ohne B.
% Formal: C = A \ B = {x | (x in A) und (x nicht in B)}
% Daraus ergibt sich für uns auf der Aufgabenstellung folgende Belegung der Argumente für delete_sublist/3:
% delete_sublist(B,A,C) [Vorsicht: die Vorkommen der ersten Liste werden aus der zweiten gelöscht!]


%
%% Korollar:
% In der vorangegangen Übung deleteall/3 haben wir bereits eine vereinfachte Version der Differenz implementiert.
% Sie ermöglicht uns die Differenzmenge von genau einem Element zu bilden.
% Formal: C = A \ {x} ist equivalent zu deleteall(x, A, C) [Vorsicht: x steht auch in deleteall für ein beliebiges Element]
% Folgend das Prädikat:

delete_all(_,[],[]).
delete_all(E,[E|T],L2):-
     delete_all(E,T,L2).

delete_all(E,[H|T1],[H|T2]):-
     E\=H,
     delete_all(E,T1,T2).
    
    
%
%%% Lösung:
% Wir haben bereits ein Prädikat, welches jeweils ein Element aus einer Liste vollständig entfernt,
% wenn wir dies nun mit rekursiver Programmierung und der Head-Tail-Zerlegung kombinieren, erhalten wir eine elegante Lösung.

% Ankerklausel: 
% Es gilt, die zu entfernende Liste ist leer/abgearbeitet und die zu bearbeitende ist gleich der Ausgabe. (erfolgreicher Durchlauf)
delete_sublist([],C,C).

% Hauptaufruf, inklusive Rekursion:
% Im Prädikatsaufruf teilen wir B in seinen Head (H) und Tail (T) auf. 
% A und C sind wie oben beschrieben.
delete_sublist([H|T], A, C) :- 
     delete_all(H, A, A_wo_H),          % Wir löschen mit delete_all das erste Element aus B (H) aus A und erhalten A_wo_H (wo = without)
     delete_sublist(T, A_wo_H, C).      % Wir rufen delete_sublist erneut auf, nun mit den restlichen Elementen von B (T) sowie A_wo_H
     

%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.

% ?- delete_sublist([a,b],[b,a,c,a,b,d],[c,d]).
% -> true.

% ?- delete_sublist([],[c,d],[c,d]).
% -> true.

% ?- delete_sublist([a,b],[],[]).
% -> true.

% ?- delete_sublist([a,b,c],[a,b,c,d],X).
% -> X = [d].


%
%% Weitere Informationen:

% -> Informationen zur Differenzmenge: https://www.mathebibel.de/differenzmenge
% -> Alternative Lösung mit find_all: https://stackoverflow.com/questions/52769929/prolog-how-do-i-delete-a-sublist-from-a-list-follow-up-question
