%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                               %
% Übung: Parser [Zusatzaufgabe]                                                 %
% Auf Seite 28 von Foliensatz 4 findest du eine Grammatik, mit der du           %
% Ableitungsbäume auf ihre Wohlgeformtheit testen kannst.                       %
%                                                                               %
% Hier im Kapitel 6 haben wir gesehen, wie wir das Prädikat append/3 in einer   %
% Grammatik zur Generierung von Sätzen einsetzen können.                        %
%                                                                               %
% Verbinde beide Ansätze zu einem Parser:                                       %
% Ziel ist ein zweistelliges Prädikat s/2, das gelingt, wenn das erste          %
% Argument ein grammatischer Satz in Form einer Wortliste ist und das zweite    %
% der Ableitungsbaum dieses Satzes.                                             %
%                                                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%
%% Erläuterung:
% Unsere Lösung beinhaltet im Gegensatz zu der Grammatik aus Foliensatz 4 Pluralformen. 
% Diese sind jedoch optional und müssen nicht zwingend eingebaut werden, sodass die 
% Grammatik aus Foliensatz 4 als Vorlage genutzt werden kann.
% Ziel ist es nun, ein Prädikat zu implementieren, welches nicht nur prüft ob eine Wortliste
% einen grammatischen Satz bildet, sondern zusätzlich auch den entsprechenden Ableitungsbaum ausgibt.
% Um die Wortlisten aus ihren einzelnen Bestandteilen zusammenzusetzen benutzen wir hier das Prädikat append/3.
% Bei append(A,B,C) werden Liste A und Liste B verknüpft, wodurch Liste C entsteht.


%
%%% Lösung:
% Für unsere Lösung müssen wir die Prädikate s/2 für die Definition eines Satzes,
% np/3 für die Definition einer Nominalphrase 
% und vp/3 für die Definition von Verbalphrasen implementieren.
% Hierbei ist das erste Argument dieser Prädikate immer eine Liste aus Worten,
% welche einen grammatischen Satz bilden und das zweite der zugehörige Ableitungsbaum.
% Die Wortlisten werden in unserer Lösung Sstring für den gesamten Satz, 
% NPstring für die Nominalphrase und VPstring für die Verbalphrase genannt.
% Die Prädikate np/3 und vp/3 haben ein Argument mehr als s/2, da hier an dritter Stelle noch der
% Numerus (NUM) hinzugefügt wird. Dieser ist notwendig, da unser Lexikon Worte im Singular und Plural enthält 
% und diese entsprechend aufeinander abgestimmt werden müssen, damit der Satz grammatikalisch korrekt ist.
% Die einzelnen Bestandteile einer NP (Determinierer und Nomen) müssen im Numerus mit
% dem Verb der VP übereinstimmen. Eine in einer VP enthaltene NP kann einen anderen Numerus haben (NUM2).

% Im letzten Schritt wird jeweils der String, welchen das Prädikat als erstes Argument nimmt (S, NP oder VP)
% aus seinen einzelnen Bestandteilen mit append/3 zusammengesetzt.

% Für vp/3 gibt es zwei Definitionen: Die erste definiert ein Verb kombiniert mit einer Nominalphrase
% für Sätze wie [die, katze, jagt, eine, maus] und die zweite definiert nur das Verb allein, 
% wodurch Sätze wie [die, katze, jagt] möglich werden. 
% Bei letzterer VP-Definition ist daher kein Zusammenfügen mit append/3 notwendig.


% Grammatik

% s/2
% s(S-Wortliste,S-Ableitungsbaum)
s(Sstring,s(NP,VP)) :-                          % s(NP,VP) ist Ableitungsbaum von Satz Sstring, 
    np(NPstring,NP,NUM),                        % wenn NP eine Nominalphrase mit Numerus NUM ist
    vp(VPstring,VP,NUM),                        % und VP eine Verbalphrase mit Numerus NUM ist
    append(NPstring,VPstring,Sstring).          % und der fertige Satz Sstring entsteht wenn NP und VP zusammengefügt werden
    
% np/3
% np(NP-Wortliste,NP-Ableitungsbaum,Numerus)
np(NPstring,np(d(D),n(N)),NUM) :-               % np(d(D),n(N)) ist Ableitungsbaum von Phrase NPstring mit Numerus NUM,
    determinierer(D,NUM),                       % wenn D ein Determinierer mit Numerus NUM ist 
    nomen(N,NUM),                               % und N ein Nomen mit Numerus NUM ist
    append([D],[N],NPstring).                   % und NPstring entsteht, wenn Det und Nomen zusammengefügt werden

% vp/3
% vp(VP-Wortliste,VP-Ableitungsbaum,Numerus)
vp(VPstring,vp(v(V),NP),NUM) :-                 % vp(v(V),NP) ist Ableitungsbaum von VPstring mit Numerus NUM,
    verb(V,NUM),                                % wenn V ein Verb mit Numerus NUM ist
    np(NPstring,NP,_NUM2),                      % und NP eine Nominalphrase mit beliebigem Numerus ist
    append([V],NPstring,VPstring).              % und VPstring entsteht wenn Verb und NPstring zusammengefügt werden

% vp/3
% vp(Verb,VP-Ableitungsbaum,Numerus)
vp([V],vp(v(V)),NUM) :-                         % vp(v(V)) ist Ableitungsbaum von dem Verb [V] mit Numerus NUM,      
    verb(V,NUM).                                % wenn V ein Verb mit Numerus NUM ist


% Lexikon

nomen(katze,sg).                                % katze ist ein Nomen im Singular
nomen(katzen,pl).                               % katzen ist ein Nomen im Plural
nomen(maus,sg).                                 % maus ist ein Nomen im Singular
nomen(maeuse,pl).                               % maeuse ist ein Nomen im Plural
determinierer(die,sg).                          % die ist ein Determinierer im Singular
determinierer(die,pl).                          % die ist ein Determinierer im Plural
determinierer(eine,sg).                         % eine ist ein Determinierer im Singular
determinierer(viele,pl).                        % viele ist ein Determinierer im Plural 
verb(klaut,sg).                                 % klaut ist ein Verb mit einem Subjekt im Singular
verb(klauen,pl).                                % klauen ist ein Verb mit einem Subjekt im Plural
verb(jagt,sg).                                  % jagt ist ein Verb mit einem Subjekt im Singular
verb(jagen,pl).                                 % jagen ist ein Verb mit einem Subjekt im Plural




%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.

% ?- s([eine, maus, jagt, viele, katzen], s(np(d(eine), n(maus)), vp(v(jagt), np(d(viele), n(katzen))))).
% -> true.

% ?- s([eine, katze, jagt, viele, maeuuse], X).
% -> X = s(np(d(eine), n(katze)), vp(v(jagt), np(d(viele), n(maeuuse)))) .

% ?- s(X,s(np(d(die), n(katze)), vp(v(klaut), np(d(die), n(maus))))).
% -> X = [die, katze, klaut, die, maus] .
