%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                       %
%% Übung: permute/2 [Zusatzaufgabe]                                     %
% Schreibe ein Prädikat permute/2, das zwei Listen akzeptiert,          %
% wenn die eine Liste aus der anderen durch Permutation der Elemente    %
% hervorgeht.                                                           %
%                                                                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%
%% Erläuterung:
% Unter Permutation versteht man in der Sprachwissenschaft die Umstellung von 
% Wörtern oder Satzgliedern innerhalb eines Satzes unter Wahrung ihrer syntaktischen Funktion.
% Unser Prädikat permute/2 nimmt zwei Listen als Eingabe 
% und soll immer "true" zurückgeben, wenn beide Listen dieselben Elemente
% in beliebiger Reihenfolge enthalten.


%
%% Korrolar:
% Für unsere Implementierung des Prädikates permute/2 benutzen wir das in der Vorlesung bereits 
% vorgestellte Prädikat delete/3, welches ein Vorkommen eines Elementes aus einer Liste entfernt.

% In der Ankerklausel von delete/3 ist das zu löschende Element X  der Head unserer Liste (Element gefunden)
% und der Tail ist identisch mit dem Ausgabe Tail. (erfolgreicher Durchlauf)

% delete/3
% delete(Term,Liste1,Liste2)
delete(X,[X|T],T).             % Ankerklausel

% Im Prädikatsaufruf teilen wir die Eingabeliste, sowie die Ausgabliste in Head und Tail auf.
% Wichtig ist hier, dass das die Heads identisch sind. Dies gilt nicht für die Tails, 
% da in T1 das Element X in unserer Ankerklausel entfernt werden soll und T2 das Ergebnis darstellt.

delete(X,[H|T1],[H|T2]):-
    delete(X,T1,T2).           % rekursiver Aufruf mit den Tails der Listen


%
%%% Lösung:
% Für die Lösung benötigen wir abgesehen von delete/3 noch das Prädikat member/2.
% member(A,B) prüft, ob Element A in Liste B vorkommt.
% Um zu prüfen, ob unsere beiden Listen Permutationen voneinader sind,
% wird zuerst getestet, ob der Head (H1) der ersten Liste ein Element der zweiten Liste (L2) ist.
% Dies muss der Fall sein, da beide Listen identische Elemente beinhalten.
% Danach wird dieser Head (H1) aus der zweiten Liste (L2) entfernt, 
% sodass als Ergebnis eine dritte Liste (L2_r) entsteht.
% Zuletzt wird überprüft ob diese neue Liste (L2_r) eine Permutation des Tails der ersten Liste (T1) ist.
% Dies ist notwendig, da T1 ohne den Head H1 dieselben Elemente wie Liste L2 ohne den Head H1 beinhalten muss.


% permute/2
% permute(?LIST, +LIST)
permute([],[]).                 % Ankerklausel

permute([H1|T1], L2) :-         % Liste [H1|T1] ist eine Permutation von Liste L2,
    member(H1,L2),              % wenn H1 ein Element der Liste L2 ist 
    delete(H1, L2, L2_r),       % und durch Löschen von H1 aus L2 Liste L2_r entsteht
    permute(T1, L2_r).          % und T1 eine Permutation von L2_r ist.


%
%% Testen (optional)
% Diese Tests kannst du manuell in deinem Prolog Interpreter durchführen.

% ?- permute([a,b,c,d],[c,d,a,b]).
% -> true.

% ?- permute([a,b,c,d],[a,b,c,d]).
% -> true.

% ?- permute(X,[a,b,c]).
% -> X = [a, b, c] ;
% -> X = [a, c, b] ;
% -> X = [b, a, c] ;
% -> X = [b, c, a] ;
% -> X = [c, a, b] ;
% -> X = [c, b, a] 

